<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2007
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* Word Processor class
*
* @author Mark Grimshaw
* 
* $Header: /cvsroot/wikindx/wikindx3/core/wp/WP.php,v 1.86 2007/04/19 13:05:45 sirfragalot Exp $
*****/
class WP
{
// Constructor
	function WP($db, $vars)
	{
		$this->db = $db;
		$this->vars = $vars;
		include_once("core/session/SESSION.php");
		$this->session = new SESSION();
		include_once("core/messages/MESSAGES.php");
		$this->messages = new MESSAGES();
		include_once("core/messages/ERRORS.php");
		$this->errors = new ERRORS();
		include_once("core/messages/SUCCESS.php");
		$this->success = new SUCCESS();
		include_once("core/messages/UTF8.php");
		$this->utf8 = new UTF8();
		include("config.php");
		if(!isset($WIKINDX_PAPERS_DIR))
// All papers are stored in the papers/ folder
			$this->dirName = "papers";
		else
			$this->dirName = $WIKINDX_PAPERS_DIR;
		if(!isset($WIKINDX_FILES_DIR))
			$this->dirFilesName = "files";
		else
			$this->dirFilesName = $WIKINDX_FILES_DIR;
	}
/*
* Check we are allowed to perform SUWP operations
* @author Mark Grimshaw
* @version 1
* @param string function name
* @return string
*/
	function gateKeep($function)
	{
		if(!$this->session->getVar('setup_write'))
		{
			include_once("core/authorize/AUTHORIZE.php");
			$authorize = new AUTHORIZE($this->db);
			return $authorize->initLogon();
		}
		$pString = '';
		$this->formFileName = $this->formDatabaseId = FALSE;
// The standard 'content.tpl' template is required for these functions
		include_once("core/template/TEMPLATE.php");
		$this->template = new TEMPLATE('content');

// Attempt a browser sanity check since HTML design mode is not available on all browsers.
// $_SERVER[HTTP_USER_AGENT]:
// Opera 8.5 gives 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.50' - not compatible.
// Konqueror gives 'Mozilla/5.0 (compatible; Konqueror/3.2; Linux) (KHTML, like Gecko)' -- not compatible.
//
// Mozilla 1.7.8 gives'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.8) Gecko/20050603 Fedora/1.7.8-1.2.1.legacy' -- compatible.
// Netscape 8 gives 'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/20050729 Netscape/8.0.3.3' - compatible.
// Firefox gives 'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.12) Gecko/20050915 Firefox/1.0.7' - compatible.
// IE 6 gives 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322)' - compatible.
		if(!$this->session->getVar("paper_browserCompatibility"))
		{
			$browserIdent = $_SERVER['HTTP_USER_AGENT'];
			if(preg_match("/Gecko|Mozilla\/4\.0 .*?compatible; MSIE 6\.0;/", $browserIdent) && 
			!preg_match("/Konqueror|Opera/", $browserIdent))
				$this->session->setVar("paper_browserCompatibility", TRUE); // compatible.
// Probably incompatible browser but user wants to try anyway
			else if($function == 'wpBrowserTry')
			{
				$this->session->setVar("paper_browserCompatibility", TRUE); // compatible or they want to try it anyway.
				$this->displayWp();
			}
			else
				return $this->displayWarning();
		}
		if($function == 'wpDisplay')
			$this->displayWp();
// revert to saved paper - ask for confirmation
		else if($function == 'wpRevertConfirm')
			$pString = $this->revertConfirmWp();
// list papers for opening one
		else if($function == 'wpListOpen')
			$pString = $this->listOpenWp();
// list papers for appending one
		else if($function == 'listAppendWp')
			$pString = $this->listOpenWp(TRUE);
// load a paper ready for appending
		else if($function == 'wpLoadAppend')
			$pString = $this->loadAppendWp();
// list for downloading/backing up.
		else if($function == 'wpList')
			$pString = $this->listWp();
		else if($function == 'wpFileDownload')
			$pString = $this->fileDownloadWp();
		else if($function == 'wpExportDownload')
			$pString = $this->exportDownloadWp();
// list papers for deleting one or more
		else if($function == 'wpListDelete')
			$pString = $this->listDeleteWp();
//confirm delete papers
		else if($function == 'wpDeleteConfirm')
			$pString = $this->deleteConfirmWp();
// delete paper(s)
		else if($function == 'wpDelete')
			$pString = $this->deleteWp();
// import paper - init
		else if($function == 'wpImportInit')
			$pString = $this->importInitWp();
// save paper dialogue (pop up)
		else if($function == 'wpSaveDialogue')
			$pString = $this->saveDialogueWp();
// save paper as dialogue (pop up)
		else if($function == 'wpSaveAsDialogue')
			$pString = $this->saveAsDialogueWp();
// save and export paper dialogue (pop up)
		else if($function == 'wpSaveExportDialogue')
			$pString = $this->saveExportDialogueWp();
		if($pString)
		{
			$this->template->setVar('body', $pString);
			return $this->template->process();
		}
		$message = FALSE;
// These functions require special SUWP templates that are closed with core/html/CLOSESUWP.php
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$fb = new FORMATBAR();
		$jsfb = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
// Call the template's template.js to load button colours.
		$cssTemplate = $this->session->getVar('setup_template');
		$jsfb .= MISC::jsExternal("templates/$cssTemplate/template.js");
// Set the paper status message
		preg_match("/<p.*>(.*)<\/p>/U", $this->errors->text("file", "suwpNotSaved"), $array);
		$message = "<span class='error'>" . $array[1] . "</span>";
		echo "<script type=\"text/javascript\" language=\"javascript\">var paperStatusMessage=\"$message\";</script>";
// new paper
		if($function == 'wpNew')
		{
			$this->session->clearArray("wp");
			$pString = $this->newWp();
		}
// open paper
		else if($function == 'wpOpen')
		{
			$pString = $this->openWp();
			$message = '';
		}
// import paper
		else if($function == 'wpImport')
			list($pString, $message) = $this->importWp();
// save paper
		else if($function == 'wpSave')
			list($pString, $message) = $this->saveWp();
// revert to saved paper
		else if($function == 'wpRevert')
			$pString = $this->revertWp();			
// Uncomment this line to use the pre-DIV SUWP
//		$pString = $jsfb . $pString;
		$this->template->setVar('body', $pString);
// Uncomment this line to use the pre-DIV SUWP
//		return $this->template->process();
		$jsfb .= $fb->printFormatBar($this->formFileName, $this->formDatabaseId, $message);
		include_once("core/html/CLOSESUWP.php");
		new CLOSESUWP($this->db, $this->template->process(), $jsfb); // exit the script here rather than go back to index.php
	}
/*
* Display a warning in case of a probably browser incompatibility
* @author Mark Grimshaw
* @version 1
*/
	function displayWarning()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = $this->errors->text('warning', 'suwpBrowserWarning');
		$pString .= MISC::p(MISC::a("link", $this->messages->text('wp', 'browserTry'), 
			"index.php?action=wpBrowserTry"));
		$this->template->setVar('body', $pString);
		return $this->template->process();
	}
/*
* Display SUWP options
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function displayWp($message = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if($message)
			$this->template->setVar('body', MISC::p($message));
		include_once("core/html/MENU.php");
		$menu = new MENU();
		$this->template->setVar('noMenu', $menu->noMenu('wp'));
		$pString = $this->template->process();
		include_once("core/html/CLOSE.php");
		new CLOSE($this->db, $pString);
	}
/*
* Form for inporting paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function importInitWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = MISC::p($this->messages->text("wp", "import2"));
		$pString .= FORM::formMultiHeader("wpImport");
		$pString .= MISC::p(FORM::fileUpload($this->messages->text("import", "file"), "import_file", 30));
		$pString .= MISC::p(FORM::formSubmit('Proceed'));
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Import paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function importWp()
	{
		if(!isset($_FILES['import_file']))
			$this->badInput($this->errors->text("file", "upload"), 'displayWp');
		$userId = $this->session->getVar('setup_userId');
// Check for file input
		if(!($text = @file_get_contents($_FILES['import_file']['tmp_name'])))
			$this->badInput($this->errors->text("file", "read"), "displayWp");
		$text = trim($text);
// Remove any extension
		$split = explode(".", $_FILES['import_file']['name']);
		if(sizeof($split) > 1)
		{
			array_pop($split);
			$title = join(".", $split);
		}
		else
			$title = $_FILES['import_file']['name'];
		$fileName = sha1($userId . $title . $text);
		$fullFileName = $this->dirName . "/" . $fileName;
// If file exists, we don't import
		if(file_exists($fullFileName))
			$this->badInput($this->errors->text("warning", "paperExists"), 'displayWp');
// Else, write file and database entry
		if(!move_uploaded_file($_FILES['import_file']['tmp_name'], $fullFileName))
			$this->badInput($this->errors->text("file", "upload"));
		$fields[] = 'hashFilename';
		$values[] = $fileName;
		$fields[] = 'userId';
		$values[] = $userId;
		$fields[] = 'filename';
		$values[] = $title;
		$fields[] = 'timestamp';
		$values[] = $this->db->formatTimestamp();
		$this->db->insert('WKX_papers', $fields, $values);
		$databaseId = $this->db->lastAutoId();
		$this->session->setVar("wp_text", base64_encode(stripslashes($text)));
		$this->session->setVar("wp_title", base64_encode(stripslashes($title)));
		$this->session->setVar("wp_filename", $fileName);
//		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$this->formFileName = $fileName;
		$this->formDatabaseId = $databaseId;
		$pString = FORMATBAR::printPaper();
		preg_match("/<p.*>(.*)<\/p>/U", $this->success->text("importWp"), $array);
		$message = "<span class='success'>" . $array[1] . "</span>";
		return array($pString, $message);
	}
/*
* List papers ready to delete one or more
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function listDeleteWp($error = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = '';
		if($error)
			$pString = $error;
		list($papers, $papersTimestamp) = $this->grabPapers();
		if(!empty($papers))
		{
			foreach($papers as $id => $title)
				$entries[$id] = $title . " (" . $papersTimestamp[$id] . ")";
			$pString .= FORM::formHeader("wpDeleteConfirm");
			$pString .= FORM::selectFBoxValueMultiple(FALSE, "id", $entries, 20, 80) . 
				MISC::br() . $this->messages->text("hint", "multiples") . MISC::br() . 
				MISC::br() . FORM::formSubmit('Confirm');
			$pString .= FORM::formEnd();
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		return $pString;
	}
/*
* Ask for confirmation of delete papers
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function deleteConfirmWp()
	{
		if(!array_key_exists('id', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), 'listDeleteWp');
		$this->session->delVar("lockDeleteWp");
		$this->template->setVar("heading", $this->messages->text("heading", "wp"));
		$userId = $this->session->getVar('setup_userId');
		include_once("core/html/HTML.php");
		$html = new HTML();
		$pString = FORM::formHeader("wpDelete");
		foreach($this->vars['id'] as $id)
		{
			$conditions[] = $this->db->tidyInput($id);
			$pString .= FORM::hidden("paperDelete_" . $id, $id);
		}
		$condition = join(" OR " . $this->db->formatField('id') . "=", $conditions);
// We do a further check here that these files really do belong to the user.
		$recordSet = $this->db->select(array("WKX_papers"), array("id", "filename"), 
			" WHERE (" . $this->db->formatField('id') . "=" . $condition . 
			") AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
		while($row = $this->db->loopRecordSet($recordSet))
			$papers[] = $html->dbToHtmlTidy($row['filename']);
		$paper = join(", ", $papers);
		$pString .= MISC::p($this->messages->text("wp", "deleteConfirm", ": $paper"));
		$pString .= MISC::br() . FORM::formSubmit('Confirm');
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Delete papers(s)
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function deleteWp()
	{
		if($this->session->getVar("lockDeleteWp"))
			$this->badInput($this->errors->text("done", "wp"), 'displayWp');
		$userId = $this->session->getVar('setup_userId');
		$ids = array();
		foreach($this->vars as $key => $value)
		{
			if(!$value)
				continue;
			if(!preg_match("/paperDelete_/", $key, $matches))
				continue;
			$ids[] = $value;
		}
		if(empty($ids))
			$this->badInput($this->errors->text("inputError", "missing"), 'listDeleteWp');
		foreach($ids as $id)
		{
			$recordSet = $this->db->select(array("WKX_papers"), array('hashFilename'), 
			" WHERE " . $this->db->formatField('id') . '=' . $this->db->tidyInput($id) . 
			" AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
			$file = $this->dirName . "/" . $this->db->fetchOne($recordSet);
			@unlink($file);
			$this->db->delete("WKX_papers", 
			" WHERE " . $this->db->formatField('id') . '=' . $this->db->tidyInput($id) . 
			" AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
		}
		$this->db->optimize("WKX_papers");
		$pString = $this->success->text("deleteWp");
		$this->session->setVar("lockDeleteWp", TRUE);
		return $this->displayWp($pString);
	}
/*
* List papers for downloading/backup
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function listWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		list($papers, $papersTimestamp) = $this->grabPapers(TRUE);
		if(!empty($papers))
		{
			foreach($papers as $id => $paperArray)
			{
				$hash = $paperArray[0];
				$title = $paperArray[1];
				$timestamp = $papersTimestamp[$id];
				$fileLinks[] = MISC::a("link", $title, 
					"index.php?action=wpFileDownload" . htmlentities("&hash=$hash&filename=$title"), "_new") . 
					"&nbsp;&nbsp;&nbsp;($timestamp)";
			}
			$pString = MISC::p($this->messages->text("wp", "backup"));
			$pString .= join(MISC::br(), $fileLinks);
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		return $pString;
	}
/*
* Download a file providing the real filename instead of the hash filename
* @author Mark Grimshaw
* @version 1
*/
	function fileDownloadWp()
	{
		if(!array_key_exists('hash', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), 'displayWp');
		$fileName = $this->vars['filename'];
		ob_start();
		header( "Content-Disposition: attachment; filename=\"$fileName\";");
		header("Expires:  0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Pragma: public");
		readfile($this->dirName . "/" . $this->vars['hash']);
		ob_end_flush();
		die;
	}
/*
* Download a paper export providing the real filename instead of the hash filename
* @author Mark Grimshaw
* @version 1
*/
	function exportDownloadWp()
	{
		if(!array_key_exists('hash', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"));
// Temp fix
		$this->vars['extension'] = 'rtf';
//
		$extension = $this->vars['extension'];
		$fileName = $this->vars['filename'];
		if($this->vars['extension'] == 'rtf')
			$type = 'text/rtf';
		ob_start();
		header("Content-type: $type");
		header( "Content-Disposition: inline; filename=\"$fileName\";" );
		header("Expires:  0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Pragma: public");
		readfile($this->dirFilesName . "/" . $this->vars['hash'] . "." . $extension);
		ob_end_flush();
		die;
	}
/*
* List papers ready to open one or append to open SUWP paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function listOpenWp($append = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = '';
		list($papers, $papersTimestamp) = $this->grabPapers();
		if($append)
		{
			$title = base64_decode($this->session->getVar('wp_title'));
			if(($key = array_search($title, $papers)) !== FALSE)
				unset($papers[$key]);
		}
		if(!empty($papers))
		{
			foreach($papers as $id => $title)
				$entries[$id] = $title . " (" . $papersTimestamp[$id] . ")";
			if($append)
			{
				$pString .= FORM::formHeader("wpLoadAppend");
				$pString .= FORM::selectFBoxValue(FALSE, "id", $entries, 20, 80) . 
					MISC::br() . FORM::formSubmit('loadAppendPaper');
			}
			else
			{
				$pString .= FORM::formHeader("wpOpen");
				$pString .= FORM::selectFBoxValue(FALSE, "id", $entries, 20, 80) . 
					MISC::br() . FORM::formSubmit('Select');
			}
			$pString .= FORM::formEnd();
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		if($append)
		{
			include_once("core/html/CLOSEPOPUP.php");
			$this->template->setVar('body', $pString);
			new CLOSEPOPUP($this->template->process());
		}
		return $pString;
	}
// load a paper ready for appending
	function loadAppendWp()
	{
		if(!array_key_exists('id', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$recordSet = $this->db->select(array("WKX_papers"), array("hashFilename", "filename"), 
			" WHERE " . $this->db->formatField("id") . "=" . $this->db->tidyInput($this->vars['id']));
		$row = $this->db->fetchRow($recordSet);
		$fullFileName = $this->dirName . '/' . $row['hashFilename'];
		$title = stripslashes($row['filename']);
		if(!($text = @file_get_contents($fullFileName)))
			$this->badInput($this->errors->text("file", "read", ": $fullFileName"), "displayWp");
		$pString = MISC::p($this->messages->text('wp', 'appendPaper'));
		$checkbox = $this->session->getVar("wp_appendSection") ? TRUE : FALSE;
		$pString .= FORM::formHeaderName("", "js", "onsubmit=\"submitForm();\"");
		$pString .= MISC::p($this->messages->text('wp', 'addSection') . "&nbsp;&nbsp;" . 
			FORM::checkbox(FALSE, "appendSection", $checkbox));
		$pString .= FORM::hidden("appendPaperText", rawurlencode($this->utf8->decodeUtf8($text)));
// Include JavaScript file
		$execute = "function submitForm() {\n  var appendPaperForm = document.getElementsByTagName('form').item('js');\n  var appendPaperString = handleAppendPaperFormRte(appendPaperForm);\n  var appendSection = handleAppendSectionFormRte(appendPaperForm);\n  window.opener.appendPaperRte(appendPaperString, appendSection);\n  window.close();\n}";
		$pString .= MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js', $execute);
		$pString .= MISC::p(FORM::formSubmit("appendPaper") . FORM::formEnd());
		include_once("core/html/CLOSEPOPUP.php");
		$this->template->setVar('body', $pString);
		new CLOSEPOPUP($this->template->process());
	}
/*
* Display blank new WP interface
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function newWp($message = FALSE)
	{
		$pString = '';
//		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if($message)
			$pString .= $message;
		$pString .= FORMATBAR::printPaper();
		return $pString;
	}
/*
* Revert to saved paper - ask for confirmation first
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function revertConfirmWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('databaseID', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$pString = MISC::p($this->messages->text("wp", "revertConfirm"));
		$pString .= FORM::formHeader("wpRevert");
		$pString .= FORM::hidden("databaseID", $this->vars['databaseID']);
		$pString .= MISC::p(FORM::formSubmit('Confirm'));
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Revert to saved paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function revertWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('databaseID', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$recordSet = $this->db->select(array("WKX_papers"), array("hashFilename", "filename"), 
			" WHERE " . $this->db->formatField("id") . "=" . $this->db->tidyInput($this->vars['databaseID']));
		$row = $this->db->fetchRow($recordSet);
		$fullFileName = $this->dirName . '/' . $row['hashFilename'];
		$title = stripslashes($row['filename']);
		if(!($text = @file_get_contents($fullFileName)))
			$this->badInput($this->errors->text("file", "read", ": $fullFileName"), "displayWp");
		$text = $this->utf8->decodeUtf8($text);
		$this->session->setVar("wp_text", base64_encode($text));
		$this->session->setVar("wp_title", base64_encode($title));
		$this->formFileName = $row['hashFilename'];
		$this->formDatabaseId = $this->vars['databaseID'];
		$pString = FORMATBAR::printPaper();
		return $pString;
	}
/*
* Open existing paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function openWp()
	{
//		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('id', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$recordSet = $this->db->select(array("WKX_papers"), array("hashFilename", "filename"), 
			" WHERE " . $this->db->formatField("id") . "=" . $this->db->tidyInput($this->vars['id']));
		$row = $this->db->fetchRow($recordSet);
		$fullFileName = $this->dirName . '/' . $row['hashFilename'];
		$title = stripslashes($row['filename']);
		if(!($text = @file_get_contents($fullFileName)))
			$this->badInput($this->errors->text("file", "read", ": $fullFileName"), "displayWp");
		$text = $this->utf8->decodeUtf8($text);
		$this->session->setVar("wp_text", base64_encode($text));
		$this->session->setVar("wp_title", base64_encode($title));
		$this->formFileName = $row['hashFilename'];
		$this->formDatabaseId = $this->vars['id'];
		$pString = FORMATBAR::printPaper();
		$this->session->setVar("wp_filename", $row['hashFilename']);
		return $pString;
	}
/*
* Save and export paper dialogue
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function saveExportDialogueWp($error = FALSE)
	{
		include_once("core/html/FORM.php");
		preg_match("/<p.*>(.*)<\/p>/U", $this->success->text("saveWP"), $array);
		$message = "<span class=\'success\'>" . $array[1] . "</span>";
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= $error;
		$pString .= FORM::formHeader("wpSave", "onsubmit=\"return updateRTESave('paperText', '$message');\"");
//		if(array_key_exists('filename', $this->vars) && $this->vars['filename'])
//			$pString .= FORM::hidden("filename", $this->vars['filename']);
		if($filename = $this->session->getVar('wp_filename'))
			$pString .= FORM::hidden("filename", $filename);
		if(array_key_exists('databaseID', $this->vars) && $this->vars['databaseID'])
			$pString .= FORM::hidden("databaseID", $this->vars['databaseID']);
		$pString .= "<input type=\"hidden\" id=\"hdnpaperText\" name=\"hdnpaperText\" value=\"\">";
		$title = base64_decode($this->session->getVar("wp_title"));
		$export = $this->session->getVar("wp_exportAndSave") ? TRUE : FALSE;
		$pString .= MISC::p(FORM::textInput($this->messages->text("wp", "paperTitle"), "title", $title, 40) . 
			FORM::formSubmit('Save'));
		$pString .= $this->exportOptions();
		$pString .= FORM::formEnd();
// Tell saveWp() that we're exporting
		$this->session->setVar("wp_exporting", TRUE);
		return $pString;
	}
/*
* Save paper dialogue
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function saveDialogueWp($error = FALSE)
	{
		include_once("core/html/FORM.php");
		preg_match("/<p.*>(.*)<\/p>/U", $this->success->text("saveWP"), $array);
		$message = "<span class=\'success\'>" . $array[1] . "</span>";
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= $error;
		$pString .= FORM::formHeader("wpSave", "onsubmit=\"return updateRTESave('paperText', '$message');\"");
//		if(array_key_exists('filename', $this->vars) && $this->vars['filename'])
		if($filename = $this->session->getVar('wp_filename'))
			$pString .= FORM::hidden("filename", $filename);
		if(array_key_exists('databaseID', $this->vars) && $this->vars['databaseID'])
			$pString .= FORM::hidden("databaseID", $this->vars['databaseID']);
		if($this->formDatabaseId)
			$pString .= FORM::hidden("databaseID", $this->formDatabaseId);
		$pString .= FORM::hidden("format", 'noExport');
		$pString .= "<input type=\"hidden\" id=\"hdnpaperText\" name=\"hdnpaperText\" value=\"\">";
		$title = base64_decode($this->session->getVar("wp_title"));
		$export = $this->session->getVar("wp_exportAndSave") ? TRUE : FALSE;
		$pString .= MISC::p(FORM::textInput($this->messages->text("wp", "paperTitle"), "title", $title, 40) . 
			FORM::formSubmit('Save'));
		$pString .= FORM::formEnd();
// Tell saveWp() that we're saving
		$this->session->delVar("wp_exporting");
		return $pString;
	}
/*
* Save paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function saveWp()
	{
//		echo '<script type="text/javascript" language="javascript">alert("closing window");</script>';
//		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!$this->writeSession())
		{
			preg_match("/<p.*>(.*)<\/p>/U", $this->errors->text("file", "suwpNotSaved"), $array);
			$message = "<span class=\'error\'>" . $array[1] . "</span>";
			echo "<script type=\"text/javascript\" language=\"javascript\">var suwpMessage=window.opener.document.getElementById('suwpMessage');suwpMessage.innerHTML=\"$message\"</script>";
			if($this->session->getVar("wp_exporting")) // exporting
				$method = "saveExportDialogueWp";
			else
				$method = "saveDialogueWp";
			$this->badInputSave($this->errors->text("inputError", "missing"), $method);
		}
		$text = stripslashes(trim($this->vars['hdnpaperText']));
		$text = $this->tidyHtml($text);
//die;
		$title = trim($this->vars['title']);
		$userId = $this->session->getVar('setup_userId');
		$this->formFileName = $fileName = sha1($userId . $title . $text);
// inserting
		if(!array_key_exists('databaseID', $this->vars))
		{
			$fields[] = 'hashFilename';
			$values[] = $fileName;
			$fields[] = 'userId';
			$values[] = $userId;
			$fields[] = 'filename';
			$values[] = $title;
			$fields[] = 'timestamp';
			$values[] = $this->db->formatTimestamp();
			$this->db->insert('WKX_papers', $fields, $values);
			$this->formDatabaseId = $databaseId = $this->db->lastAutoId();
		}
// updating
		else
		{
			$updateArray['hashFilename'] = $fileName;
			$updateArray['filename'] = $title;
			$updateArray['timestamp'] = $this->db->formatTimestamp();
			$this->formDatabaseId = $databaseId = $this->vars['databaseID'];
			$this->db->update('WKX_papers', $updateArray, 
				" WHERE " . $this->db->formatField('id') . "=" . $this->db->tidyInput($databaseId));
		}
		$fullFileName = $this->dirName . '/' . $fileName;
		if(!$fp = fopen("$fullFileName", "w"))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		if(!$text)
			$text = ' '; // fputs won't write empty string.
		else
			$text = $this->utf8->encodeUtf8($text);
		if(!fputs($fp, $text))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		fclose($fp);
// if this is a re-save, remove old hashed file from folder  if it's not the same
		if(array_key_exists('filename', $this->vars) && 
			($this->vars['filename'] != $fileName) && 
			file_exists($this->dirName . '/' . $this->vars['filename']))
				unlink($this->dirName . '/' . $this->vars['filename']);
		$this->session->setVar("wp_filename", $fileName);
// bomb out here for now....
		$pString = '';
// do we also want to export to RTF?
		if($exportRtfLink = $this->exportRtf($fileName, $title, $text))
		{
			$exportRtfLink = str_replace('"', "'", $exportRtfLink);
			echo "<script type=\"text/javascript\" language=\"javascript\">var fileCell=window.opener.document.getElementById('exportFile');fileCell.innerHTML=\"$exportRtfLink\";window.close();</script>";
		}
		else
			echo "<script type=\"text/javascript\" language=\"javascript\">window.close();</script>";
	}
/*
* Attempt to tidy up HTML
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function tidyHtml($text)
	{
/* This removed until a serious bug that results in loss of text data can be fixed.  This bug usually appears when an image, previously justified, is then centred
resulting in nested divs.
*/
// Nested DIVS occur (in Mozilla) usually where a user selects all text across multiple paragraphs, aligns that and then proceeds to align internal paragraphs.
// Can also occur when copying and pasting blocks of text within the same text (for some *&^@$*&%^! reason, half the div is copied and pasted in some cases!!!!).
// IE uses <p> ... </p> instead of <div> ... </div>
//		$text = preg_replace("/<p /si", "<div ", $text);
//		$text = preg_replace("/<\/p>/si", "</div>", $text);
//print "ORIGINAL: "; print htmlentities($text) . "<P>";
		$textArray = preg_split("/(<div.*[>]+.*)/Usi", 
			$text, -1, PREG_SPLIT_DELIM_CAPTURE|PREG_SPLIT_NO_EMPTY);
		if(sizeof($textArray) > 2)
			$text = $this->tidyDiv($textArray);
// Convert multiple whitespace characters to one
		$text = preg_replace("/ {2}/s", ' ', $text);
// Ensure spaces come after a HTML element - in most cases this will be what is wanted.
		$text = preg_replace("/ <\/(.*)>/Uis", "</$1> ", $text);
		return $text;
	}
/*
* Attempt to tidy up nested DIVs
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function tidyDiv($textArray)
	{
//print "MATCH: "; 
//foreach($textArray as $value)
//	print htmlentities($value) . "<BR>END ELEMENT<BR>";
//print "<P>";
		$newString = '';
		while($textArray)
		{
			$element = array_shift($textArray);
			if(sizeof($textArray))
			{
				$nextElement = array_shift($textArray);
				if(preg_match("/^<div/", $nextElement, $null))
				{
//print "0. ELEMENT: " . htmlentities($element) . "<BR>";
//print "0. NEXT ELEMENT: " . htmlentities($nextElement) . "<P>";
					$this->divOpen[] = $element;
					$newString .= $element . "</div>";
					$element = $nextElement;
					$nextElement = array_shift($textArray);
				}
				$newString .= $this->reduceDivs($element, $nextElement);
			}
			else
				$newString .= $element;
		}
		unset($textArray); // clear memory
//print "POST DIV: " . htmlentities($newString) . "<P>";
		return $newString;
	}
	function reduceDivs($element, $nextElement)
	{
		if(substr_count($nextElement, "</div>") == 1)
		{
//print "1. ELEMENT: " . htmlentities($element) . "<BR>";
//print "1. NEXT ELEMENT: " . htmlentities($nextElement) . "<P>";
			$newString = $element . $nextElement;
		}
		else if(substr_count($nextElement, "</div>") == 0)
		{
//print "2. ELEMENT: " . htmlentities($element) . "<BR>";
//print "2. NEXT ELEMENT: " . htmlentities($nextElement) . "<P>";
			$newString = $element . $nextElement . "</div>"; // e.g. <div...>something</div>
// Remove superfluous DIVs
//			$newString = preg_replace_callback("/(<div.*[>]+)(.*)(<\/div[>]+)/Usi", 
//				array($this, "divCallback"), $newString);
			$this->divOpen[] = $element;
		}
		else // presumably block of text after DIV has opened with more than one '</div>' in it
		{
//print "3. ELEMENT: " . htmlentities($element) . "<BR>";
//print "3. NEXT ELEMENT: " . htmlentities($nextElement) . "<P>";
			$split = split("</div>", $nextElement);
			$newString = '';
			if(isset($this->divOpen))
				array_unshift($this->divOpen, $element);
			else
				$this->divOpen[] = $element;
			while($split)
			{
				if(empty($this->divOpen))
					$openDiv = "<div>";
				else
					$openDiv = array_shift($this->divOpen);
//				$newString .= "</div>" . array_shift($split) . $openDiv;
				$newString .= $this->reduceDivs($openDiv, array_shift($split));
			}
//			$openDiv = $element;
//			$newString = array_shift($split);
// Remove superfluous DIVs
//			$newString = preg_replace_callback("/(<div.*[>]+)(.*)(<\/div[>]+)/Usi", 
//				array($this, "divCallback"), $newString);
		}
		return $newString;
	}
	function divCallback($matches)
	{
		if(!$matches[2])
			return '';
		array_shift($matches);
		return join('', $matches);
	}
/*
* Get all this user's papers' titles and ids
* @author Mark Grimshaw
* @version 1
* @param string BOOLEAN
* @return array
*/
	function grabPapers($download = FALSE)
	{
		$deleted = FALSE;
		include_once("core/html/HTML.php");
		$html = new HTML();
		$papers = $papersTimestamp = $hashCheck = $deleteMe = array();
		$fields = array("id", "hashFilename", "filename", "timestamp");
		$userId = $this->session->getVar('setup_userId');
		$recordSet = $this->db->select(array("WKX_papers"), $fields, 
			" WHERE " . $this->db->formatField("userId") . "=" . $this->db->tidyInput($userId) . 
			" ORDER BY " . $this->db->formatField('timestamp') . " DESC");
		while($row = $this->db->loopRecordSet($recordSet))
		{
			if($download)
			{
				$papers[$row['id']][0] = $row['hashFilename'];
				$papers[$row['id']][1] = $html->dbToFormTidy($row['filename']);
				$papersTimestamp[$row['id']] = $html->dbToFormTidy($row['timestamp']);
// In case of multiple entries for same hashed file (see below)
				if(array_key_exists($row['hashFilename'], $hashCheck))
					$deleteMe[] = $row['id'];
				else
					$hashCheck[$row['hashFilename']] = TRUE;
			}
			else
			{
				$papers[$row['id']] = $html->dbToFormTidy($row['filename']);
				$papersTimestamp[$row['id']] = $html->dbToFormTidy($row['timestamp']);
// In case of multiple entries for same hashed file (see below)
				if(array_key_exists($row['hashFilename'], $hashCheck))
					$deleteMe[] = $row['id'];
				else
					$hashCheck[$row['hashFilename']] = TRUE;
			}
// There's a bug somewhere I haven't been able to track down that saves multiple versions of the same hashed file in the db table.  This is a fix to prevent 
// the user noticing if another db entry of the same hash filename has previously been deleted...
// MG -- 8/Aug/2006.  The bug probably arises from a missing databaseId that is a parameter passed to the javascript save dialogue routines in FORMATBAR.  If a file is 
// opened, this id is correctly passed.  If creating a new file, there is no id to pass.  This is fine for the first save, but for subsequent saves, there is still never and id
// to pass and so multiple files get inserted into the db.  Need to figure out how to account for this.........
			$fullFileName = $this->dirName . '/' . $row['hashFilename'];
// Remove db entries if file no longer exists
			if(!file_exists("$fullFileName"))
			{
				$this->db->delete("WKX_papers", " WHERE " . $this->db->formatField('hashFilename') . '=' . 
					$this->db->tidyInput($row['hashFilename']));
				$deleted = TRUE;
			}
		}
// If there are multiple db entries of the same hash filename but the file exists, remove all but the latest from the database
		foreach($deleteMe as $id)
		{
			$this->db->delete("WKX_papers", " WHERE " . $this->db->formatField('id') . '=' . 
				$this->db->tidyInput($id));
			$deleted = TRUE;
		}
//print "PAPERS: "; print_r($papers); print "<P>";
//print "DELETEME: "; print_r($deleteMe); 
		if($deleted)
			$this->db->optimize("WKX_papers");
		return array($papers, $papersTimestamp);
	}
// Print paper export options
	function exportOptions()
	{
		include_once("core/styles/LOADSTYLE.php");
		$formats = array(
//				'noExport' => $this->messages->text("wp", "noExport"),
				'rtf' => $this->messages->text("wp", "exportRtf"),
			);
		$pString = TABLE::tableStart();
		$pString .= TABLE::trStart();
// Save or save and export
		if($format = $this->session->getVar("exportPaper_format"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "export"), 
			"exportPaper_format", $formats, $format, 1));
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "export"), 
			"exportPaper_format", $formats, 'rtf', 1));
// Paper size
		$sizes = array(
			'letter' => "Letter",
			'A4' => "A4",
			'A5' => "A5",
			'legal'	=>	'Legal',
			'executive'	=>	'Executive',
		);
		if($size = $this->session->getVar("exportPaper_paperSize"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "paperSize"), 
			"exportPaper_paperSize", $sizes, $size, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "paperSize"), 
			"exportPaper_paperSize", $sizes, 'A4', 3), FALSE, FALSE, "bottom");
// Line spacing for paper body
		$spaces = array(
			'singleSpace' => $this->messages->text("wp", "singleSpace"),
			'oneHalfSpace' => $this->messages->text("wp", "oneHalfSpace"),
			'doubleSpace' => $this->messages->text("wp", "doubleSpace"),
		);
		$fontSizeArray = array(9,11,12,16,20,28,36);
		if($space = $this->session->getVar("exportPaper_spacePaper"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpacePaper"), 
			"exportPaper_spacePaper", $spaces, $space, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpacePaper"), 
			"exportPaper_spacePaper", $spaces, 'singleSpace', 3), FALSE, FALSE, "bottom");
		$sectionFtRestart = $this->session->getVar("exportPaper_sectionFtRestart") ? TRUE : FALSE;
		$pString .= TABLE::td($this->messages->text("wp", "sectionFtRestart") . ":&nbsp;&nbsp;" . 
			FORM::checkbox(FALSE, "exportPaper_sectionFtRestart", $sectionFtRestart));
		
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();
	
// bibliographic/citation export style
		$styles = LOADSTYLE::loadDir("styles/bibliography");
		if($style = $this->session->getVar("exportPaper_style"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("config", "style"), 
			"exportPaper_style", $styles, $style, 4), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("config", "style"), 
			"exportPaper_style", $styles, $this->session->getVar("setup_style"), 4), FALSE, FALSE, "bottom");
// Line spacing for appended bibliography
		if($space = $this->session->getVar("exportPaper_spaceBib"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceBib"), 
			"exportPaper_spaceBib", $spaces, $space, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceBib"), 
			"exportPaper_spaceBib", $spaces, 'singleSpace', 3), FALSE, FALSE, "bottom");
// Indentation for appended bibliography
		$indents = array(
				'indentNone' => $this->messages->text("wp", "indentNone"),
				'indentAll' => $this->messages->text("wp", "indentAll"),
				'indentFL' => $this->messages->text("wp", "indentFL"),
				'indentNotFL' => $this->messages->text("wp", "indentNotFL"),
			);
		if($indent = $this->session->getVar("exportPaper_indentBib"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentBib"), 
			"exportPaper_indentBib", $indents, $indent, 4), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentBib"), 
			"exportPaper_indentBib", $indents, 'indentNone', 4), FALSE, FALSE, "bottom");
		$pString .= TABLE::td("&nbsp;");
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
	
		$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart();
		
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();
		$pString .= TABLE::td("&nbsp;");
// Line spacing for footnotes
		if($space = $this->session->getVar("exportPaper_spaceFt"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceFt"), 
			"exportPaper_spaceFt", $spaces, $space, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceFt"), 
			"exportPaper_spaceFt", $spaces, 'singleSpace', 3), FALSE, FALSE, "bottom");
// Indentation for footnotes
		$indents = array(
				'indentNone' => $this->messages->text("wp", "indentNone"),
				'indentAll' => $this->messages->text("wp", "indentAll"),
				'indentFL' => $this->messages->text("wp", "indentFL"),
				'indentNotFL' => $this->messages->text("wp", "indentNotFL"),
			);
		if($indent = $this->session->getVar("exportPaper_indentFt"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentFt"), 
			"exportPaper_indentFt", $indents, $indent, 4), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentFt"), 
			"exportPaper_indentFt", $indents, 'indentNone', 4), FALSE, FALSE, "bottom");
// footnote font size
		if(($fontSize = $this->session->getVar("exportPaper_fontSizeFt")) !== FALSE)
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "fontSizeFt"), 
			"exportPaper_fontSizeFt", $fontSizeArray, $fontSize, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "fontSizeFt"), 
			"exportPaper_fontSizeFt", $fontSizeArray, 0, 3), FALSE, FALSE, "bottom"); // default 9pt
// page numbering
		$pns = array(
				1 => $this->messages->text("wp", "pageNumberFooter"),
				2 => $this->messages->text("wp", "pageNumberHeader"),
				0 => $this->messages->text("wp", "pageNumberNone"),
			);
		if(($pn = $this->session->getVar("exportPaper_pageNumber")) !== FALSE)
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "pageNumber"), 
			"exportPaper_pageNumber", $pns, $pn, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "pageNumber"), 
			"exportPaper_pageNumber", $pns, 1, 3), FALSE, FALSE, "bottom");
// page numbering alignment
		$pns = array(
				0 => $this->messages->text("wp", "pageNumberAlignCentre"),
				1 => $this->messages->text("wp", "pageNumberAlignLeft"),
				2 => $this->messages->text("wp", "pageNumberAlignRight"),
			);
		if(($pn = $this->session->getVar("exportPaper_pageNumberAlign")) !== FALSE)
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "pageNumberAlign"), 
			"exportPaper_pageNumberAlign", $pns, $pn, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "pageNumberAlign"), 
			"exportPaper_pageNumberAlign", $pns, 0, 3), FALSE, FALSE, "bottom");
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
	
		$pString .= TABLE::tdEnd() . TABLE::trEnd() . TABLE::trStart() . TABLE::tdStart();
		$pString .= MISC::br();
		$pString .= TABLE::tableStart();
		$pString .= TABLE::trStart();

// Indent large quotations
		$indentQuoteWords = $this->session->getVar("exportPaper_indentQuoteWords");
		$pString .= TABLE::td(FORM::textInput($this->messages->text("wp", "indentQuoteWords"), 
			"exportPaper_indentQuoteWords", $indentQuoteWords, 4));
// Line spacing for indented quotations
		if($space = $this->session->getVar("exportPaper_spaceIndentQ"))
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceIndentQ"), 
			"exportPaper_spaceIndentQ", $spaces, $space, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "lineSpaceIndentQ"), 
			"exportPaper_spaceIndentQ", $spaces, 'singleSpace', 3), FALSE, FALSE, "bottom");
		if(($fontSize = $this->session->getVar("exportPaper_indentQuoteFontSize")) !== FALSE)
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentQuoteFontSize"), 
			"exportPaper_indentQuoteFontSize", $fontSizeArray, $fontSize, 3), FALSE, FALSE, "bottom");
		else
			$pString .= TABLE::td(FORM::selectedBoxValue($this->messages->text("wp", "indentQuoteFontSize"), 
			"exportPaper_indentQuoteFontSize", $fontSizeArray, 2, 3), FALSE, FALSE, "bottom"); // default 12pt
		$indentQuoteMarks = $this->session->getVar("exportPaper_indentQuoteMarks") ? TRUE : FALSE;
		$pString .= TABLE::td($this->messages->text("wp", "indentQuoteMarks") . ":&nbsp;&nbsp;" . 
			FORM::checkbox(FALSE, "exportPaper_indentQuoteMarks", $indentQuoteMarks));
		$pString .= TABLE::trEnd();
		$pString .= TABLE::tableEnd();
//		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* export paper to RTF
* @author Mark Grimshaw
* @version 1
* @param string hashed filename
* @param string real filename
* @param string input text
* @return string hyperlink to RTF file
*/
	function exportRtf($fileName, $title, $text)
	{
		if(!array_key_exists('exportPaper_format', $this->vars))
			return FALSE;
		else
			$this->session->setVar('exportPaper_format', $this->vars['exportPaper_format']);
		if($this->vars['exportPaper_format'] == 'noExport')
			return FALSE;
		if(array_key_exists('exportPaper_style', $this->vars))
			$this->session->setVar('exportPaper_style', $this->vars['exportPaper_style']);
		if(array_key_exists('exportPaper_paperSize', $this->vars))
			$this->session->setVar('exportPaper_paperSize', $this->vars['exportPaper_paperSize']);
		else
			$this->session->setVar('exportPaper_paperSize', 'A4');
		if(array_key_exists('exportPaper_spacePaper', $this->vars))
			$this->session->setVar('exportPaper_spacePaper', $this->vars['exportPaper_spacePaper']);
		else
			$this->session->setVar('exportPaper_spacePaper', 'singleSpace');
		if(array_key_exists('exportPaper_sectionFtRestart', $this->vars))
			$this->session->setVar('exportPaper_sectionFtRestart', $this->vars['exportPaper_sectionFtRestart']);
		else
			$this->session->setVar('exportPaper_sectionFtRestart', FALSE); // default no restart
		if(array_key_exists('exportPaper_spaceBib', $this->vars))
			$this->session->setVar('exportPaper_spaceBib', $this->vars['exportPaper_spaceBib']);
		else
			$this->session->setVar('exportPaper_spaceBib', 'singleSpace');
		if(array_key_exists('exportPaper_indentBib', $this->vars))
			$this->session->setVar('exportPaper_indentBib', $this->vars['exportPaper_indentBib']);
		else
			$this->session->setVar('exportPaper_indentBib', 'none');
		if(array_key_exists('exportPaper_spaceFt', $this->vars))
			$this->session->setVar('exportPaper_spaceFt', $this->vars['exportPaper_spaceFt']);
		else
			$this->session->setVar('exportPaper_spaceFt', 'singleSpace');
		if(array_key_exists('exportPaper_indentFt', $this->vars))
			$this->session->setVar('exportPaper_indentFt', $this->vars['exportPaper_indentFt']);
		else
			$this->session->setVar('exportPaper_indentFt', 'none');
		if(array_key_exists('exportPaper_fontSizeFt', $this->vars))
			$this->session->setVar('exportPaper_fontSizeFt', 
			$this->vars['exportPaper_fontSizeFt']);
		else
			$this->session->setVar('exportPaper_fontSizeFt', 1);
		if(array_key_exists('exportPaper_pageNumber', $this->vars))
			$this->session->setVar('exportPaper_pageNumber', $this->vars['exportPaper_pageNumber']);
		else
			$this->session->setVar('exportPaper_pageNumber', 1);
		if(array_key_exists('exportPaper_pageNumberAlign', $this->vars))
			$this->session->setVar('exportPaper_pageNumberAlign', $this->vars['exportPaper_pageNumberAlign']);
		else
			$this->session->setVar('exportPaper_pageNumberAlign', 0);
		if(array_key_exists('exportPaper_indentQuoteWords', $this->vars))
			$this->session->setVar('exportPaper_indentQuoteWords', $this->vars['exportPaper_indentQuoteWords']);
		else
			$this->session->delVar('exportPaper_indentQuoteWords');
		if(array_key_exists('exportPaper_spaceIndentQ', $this->vars))
			$this->session->setVar('exportPaper_spaceIndentQ', $this->vars['exportPaper_spaceIndentQ']);
		else
			$this->session->setVar('exportPaper_spaceIndentQ', 'singleSpace');
		if(array_key_exists('exportPaper_indentQuoteFontSize', $this->vars))
			$this->session->setVar('exportPaper_indentQuoteFontSize', 
			$this->vars['exportPaper_indentQuoteFontSize']);
		else
			$this->session->setVar('exportPaper_indentQuoteFontSize', 1);
		if(array_key_exists('exportPaper_indentQuoteMarks', $this->vars))
			$this->session->setVar('exportPaper_indentQuoteMarks', $this->vars['exportPaper_indentQuoteMarks']);
		else
			$this->session->delVar('exportPaper_indentQuoteMarks');
// Line spacing of main paper body
		if($this->vars['exportPaper_spacePaper'] == 'oneHalfSpace')
			$this->lineSpacing = '\\sl360\\slmult1';
		else if($this->vars['exportPaper_spacePaper'] == 'doubleSpace')
			$this->lineSpacing = '\\sl480\\slmult1';
		else
			$this->lineSpacing = '';
// Line spacing of indented quotations
		if($this->vars['exportPaper_spaceIndentQ'] == 'oneHalfSpace')
			$this->lineSpacingIndentQ = '\\sl360\\slmult1';
		else if($this->vars['exportPaper_spaceIndentQ'] == 'doubleSpace')
			$this->lineSpacingIndentQ = '\\sl480\\slmult1';
		else
			$this->lineSpacingIndentQ = '\\sl240\\slmult1'; // single
		include_once("core/wp/WPRTF.php");
		$this->rtf = new WPRTF($this->lineSpacing, $this->lineSpacingIndentQ);
		$output = $this->rtf->openRtf();
		$text = $this->rtfParse($text);
		if(isset($this->rtf->fontBlock))
			$output .= $this->rtf->fontBlock;
		unset($this->rtf->fontBlock);
		$this->rtf->closeListTable();
		$output .= $this->rtf->listTable;
		$output .= $this->styleSheet;
// Set default font and size and black colour and left justification
		$output .= "{\\f0\\fs24\n\\s1\\cf1\\ql\\li1\\fi1" . $this->lineSpacing . "\n";
		$output .= ltrim($text);
		$output .= "\n\\par\n}";
		$output .= $this->rtf->closeRtf();
//print "$output<P>";
// Write to files/ folder
		$fullFileName = $this->dirFilesName . '/' . $fileName . '.rtf';
		if(!$fp = fopen("$fullFileName", "w"))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		if(!fputs($fp, $this->rtf->utf8_2_unicode($output)))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		fclose($fp);
// Write exported hashed filename to session
		if($sessVar = $this->session->getVar('paperExports'))
			$sessArray = unserialize($sessVar);
		else
			$sessArray = array();
		if(array_search($fileName, $sessArray) === FALSE)
		{
			$sessArray[$fileName] = $title . '.rtf';
			$this->session->setVar('paperExports', serialize($sessArray));
		}
// Perform some system admin
		include_once("core/file/FILE.php");
		$file = new FILE($this->db);
		$file->tidyFiles();
// Create download link and return it
		$titleUrl = rawurlencode($title);
		return MISC::a("link", $title . '.rtf', "index.php?action=wpExportDownload" . 
			htmlentities("&hash=$fileName&filename=$titleUrl&extension=rtf"), "_new");
	}
/*
* parse BBCode and wikindx [...] code to RTF tags
* @author Mark Grimshaw
* @version 1
* @param string input text
* @return string parsed text
*/
	function rtfParse($text)
	{
		include_once("core/cite/CITE.php");
		$cite = new CITE($this->db, $this->vars);
// Deal with potential RTF control characters first
		$text = str_replace("\\", "\\\\", $text);
		$text = str_replace("{", "\\{", $text);
		$text = str_replace("}", "\\}", $text);
// Replace CR and newline characters and combinations with a single space
		$text = preg_replace("/(\r\n)+|(\n\r)+|\n+|\r+/", ' ', $text);
// Simple substitutions - italics and bold sometimes come in as standard HTML tags if they were copy 'n' pasted from elsewhere.
		$pattern = array(
//							"/\s*<br>\s*<\/p>/si",
//							"/\s*(.*?)\s*<br>/si",
							"/<hr>/si",
							"/<p>\s*(.*?)\s*<\/p>\s*/si",
							"/<sup>(.*?)<\/sup>/si",
							"/<sub>(.*?)<\/sub>/si",
							"/&amp;/si",
							"/\s*<br.*?>\s*/si",
							"/<em>(.*?)<\/em>/si",
							"/<i>(.*?)<\/i>/si",
							"/<strong>(.*?)<\/strong>/si",
							"/<b>(.*?)<\/b>/si",
							"/&lt;/si",
							"/&gt;/si",
						);
		$change = array(
//							"</p>",
//							"\\s1\\cf1$1__WIKINDX__NEWLINEPAR__", // temporary replace
							"__WIKINDX__SECTION__",
							"\\s1\\cf1$1__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__", // temporary replace
							"{\\super $1}",
							"{\\sub $1}",
							"&",
							"__WIKINDX__NEWLINEPAR__",
							"{\\i $1}",
							"{\\i $1}",
							"{\\b $1}",
							"{\\b $1}",
							"\\u60 ?",
							"\\u62 ?",
						);
		$text = preg_replace($pattern, $change, $text);
// handle <font>...</font>
		$text = $this->rtf->parseFont($text);
// handle <span>...</span>
		$text = $this->rtf->parseSpan($text);
		$text = $this->rtf->createLists($text);
// Handle tables
		$text = $this->rtf->createTables($text);
// Handle images
		$text = preg_replace_callback("/<img.*[>]+/Usi",
			array($this->rtf, "imageCallback"), $text);
// URL + emails
		$text = $this->rtf->createEmail($text);
		$text = $this->rtf->createFancyUrl($text);
// Handle <div>...</div>
		$text = $this->rtf->parseDiv($text);
//print "FINAL: " . htmlentities($text) . "<P>";
// Ensure there are no hanging HTML tags (some browsers produce untidy HTML or superfluous HTML)
		$patterns = array(
			"/<\/div>/iU",
			"/<\/span>/iU",
			"/<\/font>/iU",
			);
		$text = preg_replace($patterns, '', $text);
// indent long quotations
		if(array_key_exists('exportPaper_indentQuoteWords', $this->vars))
		{
			$quoteIndentWords = $this->vars['exportPaper_indentQuoteWords'];
			if(is_numeric($quoteIndentWords))
			{
				settype($quoteIndentWords, "integer");
				if(array_key_exists('exportPaper_indentQuoteFontSize', $this->vars))
					$indentQuoteFontSize = $this->vars['exportPaper_indentQuoteFontSize'];
				else
					$indentQuoteFontSize = 1; // font size 11
				if(array_key_exists('exportPaper_indentQuoteMarks', $this->vars))
					$indentQuoteMarks = TRUE;
				else
					$indentQuoteMarks = FALSE;
				$text = $this->rtf->indentQuotations($text, $quoteIndentWords, $indentQuoteFontSize, 
					$indentQuoteMarks);
// Perhaps footnote immediately follows (so should be part of indent) -- NB indentQ() method in WPRTF.php
//print "TEXT1: $text<P>";
				$text = preg_replace_callback("/(\[\/cite\][.,:;?!])(\s*__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR____WIKINDX__QUOTEINDENTREMOVESPACE__\}*)(\[footnote\].*\[\/footnote\])/Usi", 
					array($this, "callback_footnoteIndent"), $text);
//print "TEXT2: $text<P>";
//				$pattern = 	"__WIKINDX__QUOTEINDENTDONE__|__WIKINDX__NEWLINE__|__WIKINDX__QUOTEINDENTREMOVESPACE__(\s*)|__WIKINDX__QUOTEINDENTREMOVESPACE__\}*(\s*)";
				$text = preg_replace("/$pattern/", "\n", $text);
			}
		}
// Deal with citations
		$text = $cite->parseCitations($text, 'rtf', FALSE, FALSE, TRUE);
//print "TEXT3: $text<P>";
		$footnoteOffsetIds = array();
// If we have no cite tags to parse, $cite->cite is not set but we still need the stylesheet so have to load CITESTYLE and CITEFORMAT anyway
		if(!$cite->cite)
		{
			include_once("core/styles/CITESTYLE.php");
			$cite = new CITESTYLE($this->db, 'rtf');
			$this->styleSheet = $cite->citeformat->createStyleSheet();
			$styleArray = $cite->citeformat->style;
		}
		else
		{
			$this->styleSheet = $cite->cite->citeformat->styleSheet;	
			$styleArray = $cite->cite->citeformat->style;
			if(isset($cite->cite->citeformat->footnoteOffsetIds))
				$footnoteOffsetIds = $cite->cite->citeformat->footnoteOffsetIds;
		}
		unset($cite);
// handles section breaks
		$text = preg_replace("/(__WIKINDX__NEWLINEPAR__)*__WIKINDX__SECTION__(__WIKINDX__NEWLINEPAR__)*/", 
			"\n\n{\\sect}\n\\sectd\n\n", $text);
// Replace temporary newlines
		$text = str_replace("__WIKINDX__NEWLINEPAR__", "\\par\n", $text);
//		$text = str_replace("__WIKINDX__NEWLINE__", "\n", $text);
		$text = str_replace("&nbsp;", " ", $text);
//		$text = str_replace("__WIKINDX__QUOTEINDENTDONE__", "\n", $text);
		$pattern = "__WIKINDX__QUOTEINDENTDONE__|__WIKINDX__NEWLINE__|__WIKINDX__QUOTEINDENTREMOVESPACE__(\s*)";
		$text = preg_replace("/$pattern/", "\n", $text);
// when user is cut 'n' pasting, superfluous codes are sometimes inserted so remove them
		$text = preg_replace("/[ ]*<span.*[>]+(.*)<\/span[>]+[ ]*/Usi", "$1", $text);
// Deal with footnotes
		if(array_key_exists('exportPaper_fontSizeFt', $this->vars))
			$fontSizeFt = $this->vars['exportPaper_fontSizeFt'];
		else
			$fontSizeFt = 0; // font size 9
		++$fontSizeFt; // IN WPRTF.php, fontSizes are index from 1 -- 7 (here, they're 0 -- 6)
		$text = $this->rtf->parseFootnotes($text, $styleArray, $footnoteOffsetIds, $fontSizeFt);
		if($this->rtf->footnoteText)
			$text .= $this->rtf->footnoteText;
		$text = str_replace('__WIKINDX__ENDNOTE__START__', '', $text);
		$text = str_replace('__WIKINDX__ENDNOTE__END__', '', $text);
		$text = str_replace(" } ", " }", $text);
// Sometimes (not sure why), single quote gets replaced as &#039; so replace it
		$text = str_replace("&#039;", "'", $text);
// sometimes, with footnote/endnote styles, __WIKINDX__ gets left behind.  Temp. FIX.
		$text = str_replace("__WIKINDX__\par\qj __", "", $text);
		return $text;
	}
// Callback for adding a footnote to an indented quotation.
	function callback_footnoteIndent($matches)
	{
		return $matches[1] . $matches[3] . $matches[2];
	}
/*
* write input to session
* @author Mark Grimshaw
* @version 1
* @return BOOLEAN
*/
	function writeSession()
	{
		$missing = FALSE;
		if(!array_key_exists('hdnpaperText', $this->vars) || !trim($this->vars['hdnpaperText']))
			$this->session->setVar("wp_text", ''); // set empty
		else
			$this->session->setVar("wp_text", base64_encode(stripslashes(trim($this->vars['hdnpaperText']))));
		if(!array_key_exists('title', $this->vars) || !trim($this->vars['title']))
			$missing = TRUE;
		else
			$this->session->setVar("wp_title", base64_encode(stripslashes(trim($this->vars['title']))));
		if($missing)
			return FALSE;
		return TRUE;
	}
/*
* Error handling
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @param string Class method
*/
	function badInput($error, $method = FALSE)
	{
		$pString = $this->$method($error);
		$this->template->setVar('body', $pString);
		$pString = $this->template->process();
		include_once("core/html/CLOSE.php");
		new CLOSE($this->db, $pString);
	}
/*
* Error handling (from the save dialogue so no menus etc. required)
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @param string Class method
*/
	function badInputSave($error, $method = FALSE)
	{
		$pString = $this->$method($error);
		$this->template->setVar('body', $pString);
		$pString = $this->template->process();
		include_once("core/html/CLOSEPOPUP.php");
		new CLOSEPOPUP($pString);
	}
}
?>
