<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2007
sirfragalot@users.sourceforge.net
**********************************************************************************/

/**
* Produce valid RTF markup.
*
* @author Mark Grimshaw
*
* NB - this produces just the bare minimum of RTF code to work in Word and OO.org
* 
* $Header: /cvsroot/wikindx/wikindx3/core/wp/WPRTF.php,v 1.57 2007/04/28 07:50:28 sirfragalot Exp $
*/
class WPRTF
{
/**
* Constructor method called by user.
*/
	function WPRTF($lineSpacing = FALSE, $lineSpacingIndentQ = FALSE)
	{
		$this->lineSpacing = $lineSpacing;
		$this->lineSpacingIndentQ = "\n" . $lineSpacingIndentQ . "\n";
		$this->footnoteText = FALSE;
		$this->tableStyle = FALSE;
		include_once("core/session/SESSION.php");
		$this->session = new SESSION();
/**
 * some defaults
 */
		$this->justify = array(
					"center"	=>	"\\qc",
					"left"		=>	"\\ql",
					"right"		=>	"\\qr",
					"justify"	=>	"\\qj",
				);
		$this->fonts = array(
					"arial",
					"courier",
					"georgia",
					"helvetica",
					"ms sans serif",
					"ms serif",
					"palatino",
					"tahoma",
					"trebuchet ms",
					"times new roman",
					"verdana",
				);
		$this->fontSizes = array(
					1	=>	18,
					2	=>	22,
					3	=>	24,
					4	=>	32,
					5	=>	40,
					6	=>	56,
					7	=>	72,
				);
/**
* Lists:
* \\ls0 => a, b ('a')
* \\ls5 => A, B ('aa')
* \\ls2 => i, ii ('i')
* \\ls3 => I, II ('ii')
* \\ls4 => 1, 2
* \\ls1 => bullets -- currently, only this one is used from this array.  Arabic numbered lists are handled in the code.
*/
		$this->lists = array(
					'aa'	=>	"\\ls5\\li720\\fi-360",
					'a'		=>	"\\ls0\\li720\\fi-360",
					'ii'	=>	"\\ls3\\li720\\fi-360",
					'i'		=>	"\\ls2\\li720\\fi-360",
					'1'		=>	"\\ls4\\li720\\fi-360",
					'bullet'	=>	"\\ls1\\li720\\fi-360",
				);
		$this->colourTable = array(
					'black' => "\\red0\\green0\\blue0;",
					'maroon' => "\\red128\\green0\\blue0;",
					'green' => "\\red0\\green128\\blue0;",
					'olive' => "\\red128\\green128\\blue0;",
					'navy' => "\\red0\\green0\\blue128;",
					'purple' => "\\red128\\green0\\blue128;",
					'teal' => "\\red0\\green128\\blue128;",
					'gray' => "\\red128\\green128\\blue128;",
					'silver' => "\\red192\\green192\\blue192;",
					'red' => "\\red255\\green0\\blue0;",
					'lime' => "\\red0\\green255\\blue0;",
					'yellow' => "\\red255\\green255\\blue0;",
					'blue' => "\\red0\\green0\\blue255;",
					'fuchsia' => "\\red255\\green0\\blue255;",
					'aqua' => "\\red0\\green255\\blue255;",
					'white' => "\\red255\\green255\\blue255;",
				);
		$this->indentL = array(
					40	=>	'\\li720',
					80	=>	'\\li1440',
					120	=>	'\\li2160',
					160	=>	'\\li2880',
					200	=>	'\\li3600',
				);
		$this->indentR = array(
					40	=>	'\\ri720',
					80	=>	'\\ri1440',
					120	=>	'\\ri2160',
					160	=>	'\\ri2880',
					200	=>	'\\ri3600',
				);
		$this->fontBlocks = array();
		$this->fontIndex = 1;
		$this->listIndex = 2;
// Default bullets - 'listid1' == bullets, 'listidN' where 'N' is greater than 1 == Arabic
// $this->listTable is closed in $this->closeListTable() below
		$this->listTable = "{\\*\\listtable\n
{\\list
{\\listlevel\\levelnfc23\\levelstartat1\\levelfollow0{\\leveltext \\'01\\u8226 ?;}{\\levelnumbers;}\\fi-360\\li720}
\\listid1}\n";
		$this->listOverrideTable = "{\\listoverride\\listid1\\ls1}\n";
	}
/**
* Create the RTF opening tag
* @return string
*/
	function openRtf()
	{
		$text = "{\\rtf1\\ansi\\ansicpg1252\n\n";
		$text .= "{\\colortbl;";
		$index = 1;
		foreach($this->colourTable as $colour => $colourCode)
		{
			$text .= $colourCode;
			$this->colours[$colour] = "\\s1\\cf$index";
			++$index;
		}
		$text .= "}\n\n";
		unset($this->colourTable);
// Paper size
		if($paperSize = $this->session->getVar("exportPaper_paperSize"))
		{
// 'letter' is default size for RTF -- presumably, 'legal' has the default width of 'letter'
// The tables always go beyond the paper width so we set each table width to 96% of standard
			if($paperSize == 'letter')
				$this->tableWidth = floor(8748 * 0.96);
			else if($paperSize == 'A4')
			{
				$text .= "\\paperw11909\\paperh16834\n\n";
				$this->tableWidth = floor(8417 * 0.96);
			}
			else if($paperSize == 'A5')
			{
				$text .= "\\paperw8395\\paperh11909\n\n";
				$this->tableWidth = floor(4903 * 0.96);
			}
			else if($paperSize == 'legal')
			{
				$text .= "\\paperh20160\n\n";
				$this->tableWidth = floor(8748 * 0.96);
			}
			else if($paperSize == 'executive')
			{
				$text .= "\\paperw10440\\paperh15120\n\n";
				$this->tableWidth = floor(6948 * 0.96);
			}
		}
		return $text . "\n\n";
	}
// Create list tables for Arabic numerals
	function createListTablesArabic()
	{
// List table
		$this->listTable .= "
{\\list
{\\listlevel\\levelnfc0\\levelstartat1\\levelfollow0{\\leveltext \\'02\\'00.;}{\\levelnumbers\\'01;}\\fi-360\\li720}\\listid" . $this->listIndex . "}\n
";
		$this->listId = "\\ls" . $this->listIndex;
		$this->listOverrideTable .= 
"{\\listoverride\\listid" . $this->listIndex . $this->listId . "}\n";
		++$this->listIndex;
	}
// Close the list table and append the listOverrideTable string
	function closeListTable()
	{
		$this->listTable .= "}\n" . "{\\listoverridetable\n" . $this->listOverrideTable . "}\n\n";
	}
/**
* Create the RTF closing tag
* @return string
*/
	function closeRtf()
	{
		return "\n}\n";
	}
/**
* Create fancy url hyperlinks
*/
	function createFancyUrl($text)
	{
		return preg_replace_callback("/<a\s*href\s*=\s*\"http:\/\/(.*)\"\s*>(.*)<\/a>/Usi",
			array($this, "setFancyUrl"), $text);
	}
/**
* Create url hyperlinks -- not used anymore
*/
	function createUrl($text)
	{
		return preg_replace_callback("/\[url\](.*)\[\/url\]/Usi",
			array($this, "setUrl"), $text);
	}
/**
* Create email hyperlinks
*/
	function createEmail($text)
	{
		return preg_replace_callback("/<a\s*href\s*=\s*\"mailto:(.*)\"\s*>(.*)<\/a>/Usi",
			array($this, "setEmail"), $text);
	}
/**
* Callback for createFancyUrl() above
*/
	function setFancyUrl($matchArray)
	{
		$url = trim($matchArray[1]);
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		return "{\\field{\\fldinst { HYPERLINK \"http://$url\" }}{\\fldrslt {\\cs1\\ul\\cf13 $text}}}__WIKINDX__NEWLINE__";
	}
/**
* Callback for createUrl() above -- not used anymore
*/
	function setUrl($matchArray)
	{
		$url = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[1]);
		return "{\\field{\\fldinst { HYPERLINK \"http://$url\" }}{\\fldrslt {\\cs1\\ul\\cf13 $url}}}__WIKINDX__NEWLINE__";
	}
/**
* Callback for createEmail() above
*/
	function setEmail($matchArray)
	{
		$email = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[1]);
		return "{\\field{\\fldinst { HYPERLINK \"mailto:$email\" }}{\\fldrslt {\\cs1\\ul\\cf13 $email}}}__WIKINDX__NEWLINE__";
	}
/**
* Set font types and create top level font blocks
*/
	function createFontBlocks($font)
	{
// Default font block
		$this->defaultFontBlock = "{\\f0\\fcharset0 Arial;}\n";
		$fontIndex = $this->setFontBlock($font);
		if(!empty($this->fontBlocks))
		{
			$this->fontBlock = "{\\fonttbl\n";
			$this->fontBlock .= $this->defaultFontBlock;
			foreach($this->fontBlocks as $index => $font)
				$this->fontBlock .= "{\\f$index\\fcharset0 $font;}\n";
			$this->fontBlock .= "}\n\n";
		}
		return $fontIndex;
	}
/**
* Set font blocks
*/
	function setFontBlock($font)
	{
		if(array_search(strtolower($font), $this->fonts) === FALSE)
			$font = 'arial';
		if(($index = array_search($font, $this->fontBlocks)) === FALSE)
		{
			$this->fontBlocks[$this->fontIndex] = $font;
			$fontIndex = $this->fontIndex;
			++$this->fontIndex;
		}
		else
			$fontIndex = $index;
		return $fontIndex;
	}
/**
* Set font sizes -- no longer in use
*/
	function createFontSizes($text)
	{
//		return preg_replace_callback("/\[size\s*=\s*(.*)\](.*)\[\/size\]/Usi",
//			array($this, "setFontSize"), $text);
		return preg_replace_callback("/<font\s*size\s*=\s*\"(.*)\"\s*>(.*)<\/font>/Usi",
			array($this, "setFontSize"), $text);
	}
/**
* Set font sizes - callback function for parseFont()
*/
	function setFontSize($matchArray)
	{
		$size = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		$fontSize = $this->fontSizes[$size];
		return "{\\fs$fontSize $text}__WIKINDX__NEWLINE__";
	}
/**
* Set font color - callback function for parseFont()
*/
	function setFontColor($matchArray)
	{
		$colour = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		if(!array_key_exists($colour, $this->colours))
			$colour = $this->colours['black'];
		else
			$colour = $this->colours[$colour];
		return "__WIKINDX__NEWLINE__$colour" . "__WIKINDX__NEWLINE__$text";
	}
/**
* Set font face - callback function for parseFont()
*/
	function setFontFace($matchArray)
	{
		$face = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		$fontIndex = $this->createFontBlocks(trim($face));
		return "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
	}
/**
* Justify text -- no longer in use
*/
	function createJustify($text)
	{
		return preg_replace_callback("/\[align\s*=\s*(.*)\](.*)\[\/align\]/Usi",
			array($this, "setJustify"), $text);
	}
/**
* Set text justification - callback for createJustify() -- no longer in use
*/
	function setJustify($matchArray)
	{
		$justify = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		if(!array_key_exists($justify, $this->justify))
			$justify = $this->justify['justify'];
		else
			$justify = $this->justify[$justify];
		return "$justify\n$text\n";
	}
/**
* Indent
* Each TAB is equivalent to 720 units of indent -- no longer in use
*/
	function createIndent($text)
	{
		$text = preg_replace_callback("/\[indent\s*=\s*(l.*)\](.*)\[\/indent\]/Usi",
			array($this, "setIndent"), $text);
		return preg_replace_callback("/\[indent\s*=\s*(r.*)\](.*)\[\/indent\]/Usi",
			array($this, "setIndent"), $text);
	}
/**
* Set text indentation - callback for createIndent() -- no longer in use
*/
	function setIndent($matchArray)
	{
		$indent = $matchArray[1];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		$indentL = $indentR = '';
		if(substr($indent, 0, 1) == 'l')
		{
			if(!array_key_exists($indent, $this->indentL))
				$indentL = $this->indentL['l1'];
			else
				$indentL = $this->indentL[$indent];
		}
		else if(substr($indent, 0, 1) == 'r')
		{
			if(!array_key_exists($indent, $this->indentR))
				$indentR = $this->indentR['r1'];
			else
				$indentR = $this->indentR[$indent];
		}
		return "$indentL$indentR\n$text\n\\par\n\\li0\\ri0\n";
	}
/**
* Indent first line
* Each TAB is equivalent to 720 units of indent
*/
	function createIndentFirstLine($text)
	{
		return preg_replace_callback("/\[indent\s*=\s*l_first_line\](.*)\[\/indent\]/Usi",
			array($this, "setIndentFirstLine"), $text);
	}
/**
* Set text indentation for first line - callback for createIndentFirstLine()
*/
	function setIndentFirstLine($matchArray)
	{
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[1]);
		return "\fi720\n$text\n\\par\n\fi0\n";
	}
/**
* UTF-8 to unicode
* returns an string of RTF unicode character codes 
 * Code adapted from opensource PHP code by Scott Reynen at:
 * http://www.randomchaos.com/document.php?source=php_and_unicode
 *
 * @parameter string $string UTF-8 encoded string
 * @return string unicode character codes
 */
	function utf8_2_unicode($string)
	{
		$unicode = array();        
		$values = array();
		$lookingFor = 1;
		for($i = 0; $i < strlen($string); $i++)
		{
			$thisValue = ord($string[$i]);
			if($thisValue < 128)
				$unicode[] = $string[$i];
			else
			{
				if(count($values) == 0)
					$lookingFor = ($thisValue < 224) ? 2 : 3;
				$values[] = $thisValue;
				if(count($values) == $lookingFor)
				{
					$number = ($lookingFor == 3) ?
						(($values[0] % 16) * 4096) + (($values[1] % 64) * 64) + ($values[2] % 64) :
						(($values[0] % 32) * 64) + ($values[1] % 64);
					$decModulus = $number % 256;
					$modulus = dechex($number % 256);
					if($decModulus < 16)
						$modulus = '0' . $modulus;
					$unicode[] = '\u' . $number . "\'$modulus";
					$values = array();
					$lookingFor = 1;
				}
			}
		}
		return join('', $unicode);
	}
/**
* Find any colours - not used anymore
*/
	function createColours($text)
	{
		return preg_replace_callback("/\[color\s*=\s*(.*)\](.*)\[\/color\]/Usi",
			array($this, "colourFormat"), $text);
	}
/**
* Format colours
*/
	function colourFormat($colours)
	{
		$colour = trim($colours[1]);
		if(!array_key_exists($colour, $this->colours))
			$colour = $this->colours['black'];
		else
			$colour = $this->colours[$colour];
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $colours[2]);
		return "__WIKINDX__NEWLINE__$colour" . "__WIKINDX__NEWLINE__$text";
	}
/**
* Find any tables
*/
	function createTables($text)
	{
		return preg_replace_callback("/\s*<table(.*)>\s*(.*)\s*<\/table>\s*/Usi",
			array($this, "tableFormat"), $text);
	}
/**
* Format Tables
*
* We've no way of knowing the browser window size from PHP so we assume a width of 1000 -- if the resultant tableWidth is higher 
* than $this->tableWidth, we limit it to $this->tableWidth.
* Must count the number of cells in each row and divide them into $tableWidth to get cell width.
*/
	function tableFormat($matchArray)
	{
		$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $matchArray[2]);
		preg_match("/width.*(\d+)px/Usi", $matchArray[1], $array);
		$tableWidth = array_key_exists(1, $array) ? $this->tableWidth * $array[1] / 1000 : $this->tableWidth;
		$tableWidth = $tableWidth > $this->tableWidth ? $this->tableWidth : floor($tableWidth);
		$row = "\n\\trowd\\trql\\trpaddft3\\trpaddt55\\trpaddfl3\\trpaddl55\\trpaddfb3\\trpaddb55\\trpaddfr3\\trpaddr55__WIKINDX__NEWLINE__";
		$cell = "\\clbrdrt\\brdrw1\\brdrcf1\\clbrdrl\\brdrs\\brdrw1\\brdrcf1\\clbrdrb\\brdrs\\brdrw1\\brdrcf1";
		$finalCell = "\\clbrdrt\\brdrs\\brdrw1\\brdrcf1\\clbrdrl\\brdrs\\brdrw1\\brdrcf1\\clbrdrb\\brdrs\\brdrw1\\brdrcf1\\clbrdrr\\brdrs\\brdrw1\\brdrcf1\\cellx" . $tableWidth . "__WIKINDX__NEWLINE__";
		preg_match_all("/\s*<tr.*>\s*(.*)\s*\<\/tr>\s*/Usi", $text, $matches);
// Count no. table cells in each row
		$output = '';
		foreach($matches[1] as $data)
		{
			$cellString = '\\intbl\n ';
			$rowString = $row;
			$numCells = preg_match_all("/\s*<td(.*)>\s*(.*)\s*\<\/td>\s*/Usi", $data, $cells);
			if(empty($cells))
				return $output;
			$cellText = $cells[2];
			$cellStyle = $cells[1];
			$width = $baseWidth = floor($tableWidth / $numCells);
			$cellNumber = 1;
//print_r($cells); print "<P>";
			foreach($cellText as $cellData)
			{
				if($cellNumber != $numCells)
					$rowString .= $cell . "\\cellx$width" . "__WIKINDX__NEWLINE__";
				else
					$rowString .= $finalCell;
				$cellData = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $cellData);
				if(preg_match("/style\s*=\s*\"(.*?)\"/si", array_shift($cellStyle), $styleMatch))
				{
					$this->tableStyle = TRUE;
					$cellData = $this->style($styleMatch[1], $cellData);
				}
				else
				{
					$cellData .= "\\cell";
					$this->tableStyle = FALSE;
				}
				$cellString .= $cellData . "__WIKINDX__NEWLINE__\\s1\\cf1";
				++$cellNumber;
				$width += $baseWidth;
			}
			$output .= $rowString . $cellString . "\\row\\pard" . $this->lineSpacing . "\n\n";
		}
		$this->tableStyle = FALSE;
		return $output;
	}
/**
* Create lists and top level \listtable
*/
	function createLists($text)
	{
		$text = preg_replace_callback("/\s*<ul.*>\s*(.*)\s*<\/ul>\s*/Usi",
			array($this, "setListU"), $text);
		$text = preg_replace_callback("/\s*<ol.*>\s*(.*)\s*<\/ol>\s*/Usi",
			array($this, "setListO"), $text);
		return $text;
	}
/**
* Callback for createLists() above - Unordered lists.
* NB - IE sometimes closes the <li> tag sometimes doesn't - bravo!
*/
	function setListU($matchArray)
	{
		$this->listType = $this->lists['bullet'];
		$text = $matchArray[1];
		$text = preg_replace("/\s*<li>\s*(.*?)\s*<\/li>\s*/si", 
			$this->listType . " \{$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*<\/li>\s*/si", 
			array($this, "listStyleCallback"), $text);
		$text = preg_replace("/\s*<li>\s*(.*?)\s*/si", 
			$this->listType . " \{$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*/si", 
			array($this, "listStyleCallback"), $text);
		return "\\par\n$text" . "\\pard" . $this->lineSpacing . "\\par\n";
	}
/**
* Callback for createLists() above -- Ordered Lists.
* NB - IE sometimes closes the <li> tag sometimes doesn't - bravo!
*/
	function setListO($matchArray)
	{
		$this->createListTablesArabic();
		$this->listType = $this->listId . "\\li720\\fi-360";
		$text = $matchArray[1];
		$text = preg_replace("/\s*<li>\s*(.*?)\s*<\/li>\s*/si", 
			$this->listType . " \{$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*<\/li>\s*/si", 
			array($this, "listStyleCallback"), $text);
		$text = preg_replace("/\s*<li>\s*(.*?)\s*/si", 
			$this->listType . " \{$1}\\par__WIKINDX__NEWLINE__", $text);
		$text = preg_replace_callback("/\s*<li\s*style\s*=\s*\"(.*?)\">\s*(.*?)\s*/si", 
			array($this, "listStyleCallback"), $text);
		return "\\par\n$text" . "\\pard" . $this->lineSpacing . "\\par\n";
	}
/*
* For the cases where a list element has style information
*/
	function listStyleCallback($match)
	{
		$text = trim($match[2]);
		$text = $this->style($match[1], $text);
		return $this->listType . " \{$text}\\par__WIKINDX__NEWLINE__";
	}
/**
* Read an image from either file or URL
*/
	function imageCallback($matchArray)
	{
		$editW = $editH = FALSE;
		if(preg_match("/src\s*=\s*\"(.*)\"/Usi", $matchArray[0], $array))
			$file = $array[1];
		else if(array_key_exists(1, $matchArray))
			$file = $matchArray[1];
		else
			return $matchArray[0]; // unable to read file so return link
		preg_match("/width.*(\d*)px/Usi", $matchArray[0], $array);
		if(array_key_exists(1, $array))
			$editW = $array[1];
		preg_match("/height.*(\d*)px/Usi", $matchArray[0], $array);
		if(array_key_exists(1, $array))
			$editH = $array[1];
// Currently just working for JPEG and PNG in both OO.org and Word. (NB - Word can open GIFs processed here.)
		$tempFile = FALSE;
		if($fp = @fopen($file, "rb")) // don't do anything on file read error
		{
			list($width, $height, $type, $atrr) = GetImageSize($file);
			if(!$editW || !$editH)
				$blipScale = "\\picscalex100\\picscaley100";
			else
			{
				$scaleX = floor(100 * ($editW/$width));
				$scaleY = floor(100 * ($editH/$height));
				$blipScale = "\\picscalex$scaleX\\picscaley$scaleY";
			}
			if($type == 2) // JPEG
				$blipType = "\\jpegblip";
			else if($type == 3) // PNG
				$blipType = "\\pngblip";
			else
			{
// Try to run imageMagick convert utility if available (probably just on *NIX) to convert to a PNG
				if(file_exists("/usr/bin/convert"))
				{
					$blipType = "\\pngblip";
					$tempFile = "files/" . session_id() . ".png";
					exec("/usr/bin/convert $file $tempFile");
				}
				else if(file_exists("/usr/bin/local/convert"))
				{
					$blipType = "\\pngblip";
					$tempFile = "files/" . session_id() . ".png";
					exec("/usr/bin/local/convert $file $tempFile");
				}
				else
					return $matchArray[0]; // unable to read file so return link
			}
			$out = "";
			if($tempFile)
			{
				fclose($fp);
				$fp = fopen($tempFile, "rb");
				@unlink($tempFile);
			}
			while (!feof($fp))
			{
				$file = fread($fp, 64);
				$out .= bin2hex($file) . "__WIKINDX__NEWLINE__";
			}
			return "{\\pict__WIKINDX__NEWLINE__" . $blipScale . $blipType . "__WIKINDX__NEWLINE__" . $out . "}";
		}
		return $matchArray[0]; // unable to read file so return link
	}
/*
* parse <font>...</font>
* @author Mark Grimshaw
* @version 1
* @param string
* @return string
*/
	function parseFont($text)
	{
		$this->isIE = $this->spanParse = FALSE;
// Gecko type
		$text = preg_replace_callback("/\s*<font\s*style\s*=\s*\"(.*?)\"(.*?)>(.*?)<\/font>\s*/si", 
			array($this, "styleCallback"), $text);
// IE type
		$this->isIE = $this->spanParse = TRUE;
		$text = preg_replace_callback("/<font\s*(.*?)\>(.*?)<\/font>/si", 
			array($this, "styleCallback"), $text);
		if(preg_match("/<font.*?\>.*?<\/font>/si", $text)) // deal with nested font tags
			$text = $this->parseFont($text);
		return $text;
	}
/*
* parse <span>...</span>
* @author Mark Grimshaw
* @version 1
* @param string
* @return string
*/
	function parseSpan($text)
	{
		$this->spanParse = TRUE;
		$this->isIE = FALSE;
		$text = preg_replace_callback("/<span\s*style\s*=\s*\"(.*?)\">(.*?)<\/span>/si", 
			array($this, "styleCallback"), $text);
// Sometimes, not all SPANs are caught - this removes them from the RTF until this problem can be fixed.
//		$text = preg_replace("/\s*<span.*?\>(.*?)<\/span>\s*/si", '$1', $text);
		if(preg_match("/<span.*?\>.*?<\/span>/si", $text)) // deal with nested span tags
			$text = $this->parseSpan($text);
		return $text;
	}
/*
* parse <div>...</div>
* @author Mark Grimshaw
* @version 1
* @param string
* @return string
*/
	function parseDiv($text)
	{
		$this->spanParse = TRUE;
		$this->isIE = FALSE;
//print "TEXT: " . htmlentities($text) . "<P>";
//		$text = preg_replace_callback("/\s*<div\s*style\s*=\s*\"(.*?)\">(.*?)<\/div>\s*/si", 
		$text = preg_replace_callback("/\s*<div\s*style\s*=\s*\"(.*?)\".*?[>]+(.*?)<\/div>\s*/si", 
			array($this, "styleCallback"), $text);
// IE prefers <P align = xxx>...</P> instead of DIV
		$text = preg_replace_callback("/\s*<p\s*align\s*=\s*\"*(.*?)\"*[>]+(.*?)<\/p>\s*/si", 
			array($this, "paraCallback"), $text);
		$text = preg_replace_callback("/\s*<p\s*style\s*=\s*\"(.*?)\".*?[>]+(.*?)<\/p>\s*/si", 
			array($this, "styleCallback"), $text);
// Sometimes, not all DIVs are caught - this removes them from the RTF until this problem can be fixed.
//		$text = preg_replace("/\s*<div.*?\>(.*?)<\/div>\s*/si", '$1', $text);
		return $text;
	}
/*
* callback for HTML style elements of font and span
* @author Mark Grimshaw
* @version 1
* @param match array
* @return string
*/
	function styleCallback($match)
	{
/*
$temp = $match;
array_shift($temp);
print "MATCH: "; 
foreach($temp as $value)
	print htmlentities($value) . "<P>";
print "<BR>";
*/
		if($this->spanParse)
			$text = $match[2];
		else
		{
			$text = trim($match[3]);
/* Not used anymore?
			if(preg_match("/size.*?(\d+)/si", trim($match[2]), $sizeArray))
			{
print "HERE<P>";
				$size = trim($sizeArray[1]);
				$fontSize = $this->fontSizes[$size];
				$text = "{\\fs$fontSize $text}__WIKINDX__NEWLINE__";
			}
*/
		}
		$newText = $this->style($match[1], $text);
		return $newText;
	}
/*
* callback for HTML style elements of <P>
* @author Mark Grimshaw
* @version 1
* @param match array
* @return string
*/
	function paraCallback($match)
	{
		$text = $match[2];
		return $this->paraStyle($match[1], $text) . "__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__";
	}
/*
* deal with style elements for DIV and SPAN
* @author Mark Grimshaw
* @version 1
* @param string
* @param string
* @return string
*/
	function style($styleString, $text)
	{
//print "STYLESTRING: $styleString<P>";
		$text = str_replace(" ", "WIKINDX_SPACE", $text);
		if($this->isIE)
			$params = split('&nbsp;', $styleString);
		else
			$params = explode(";", $styleString);
//print_r($params); print "<P>";
		$justify = $pixelsR = $pixelsL = $indentL = $indentR = FALSE;
		$slashCellAdded = FALSE;
//print_r($params); print ": $text<P>";
		foreach($params as $param)
		{
			if(!$param)
				continue;
			if($this->isIE)
				$splitParam = explode("=", $param);
			else
				$splitParam = explode(":", $param);
			if(!array_key_exists(1, $splitParam)) // not recognised - usually result of pasting from Office etc.
				continue;
//print "PARAM: $param<BR>";
//print "#" . $splitParam[0] . "# | #" . $splitParam[1] . "#<P>";
			$param0 = strtolower(trim($splitParam[0]));
			$param1 = trim(str_replace('"', '', $splitParam[1]));
//print html_entity_decode("param0: $param0; param1: $param1<P>");
//print "$param0<P>";
			if(($param0 == 'font-weight') && ($param1 == 'bold'))
				$text = "{\\b\n$text}";
			else if(($param0 == 'font-weight') && ($param1 == 'normal'))
				$text = "{$text}";
			else if(($param0 == 'text-decoration') && ($param1 == 'underline'))
				$text = "{\\ul $text}";
			else if(($param0 == 'font-style') && ($param1 == 'italic'))
				$text = "{\\i\n$text}";
			else if($param0 == 'color')
			{
				$colour = $param1;
				if(!array_key_exists($colour, $this->colours))
					$colour = $this->colours['black'];
				else
					$colour = $this->colours[$colour];
//				$text = preg_replace("/^__WIKINDX__NEWLINEPAR__|__WIKINDX__NEWLINEPAR__$/", '', $text);
				$text = "__WIKINDX__NEWLINE__\{$colour" . "__WIKINDX__NEWLINE__$text}";
			}
			else if($param0 == 'font-family')
			{
				$fontIndex = $this->createFontBlocks($param1);
				$text = "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'face')
			{
				$fontIndex = $this->createFontBlocks($param1);
				$text = "{\\f$fontIndex $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'size')
			{
				$fontSize = $this->fontSizes[$param1];
				$text = "{\\fs$fontSize $text}__WIKINDX__NEWLINE__";
			}
			else if($param0 == 'text-align')
				$justify = $param1;
			else if($param0 == 'margin-left')
			{
				if(preg_match("/(\d+)/", $param1, $array))
					$pixelsL = $array[1];
			}
			else if($param0 == 'margin-right')
			{
				if(preg_match("/(\d+)/", $param1, $array))
				{
					$pixelsR = $array[1];
					if(!array_key_exists($pixelsR, $this->indentR))
						$indentR = $this->indentR[40];
					else
						$indentR = $this->indentR[$pixelsR];
				}
			}
		}
// Although it works in OO.org, for Word, justification needs to be outside {...} and therefore not mixed up with bold, italics etc.  so deal with it last.
		if($justify)
		{
			if(!array_key_exists($justify, $this->justify))
				$justify = $this->justify['justify'];
			else
				$justify = $this->justify[$justify];
			if($this->tableStyle)
			{
				$text = "\{$justify\n$text \\cell}\n";
				$slashCellAdded = TRUE;
			}
			else
				$text = "$justify\n$text\n";
		}
		if($this->tableStyle && !$slashCellAdded)
			$text .= "\\cell";
		if($pixelsL)
		{
			if(!array_key_exists($pixelsL, $this->indentL))
				$indentL = $this->indentL[40];
			else
				$indentL = $this->indentL[$pixelsL];
		}
		if($pixelsR)
		{
			if(!array_key_exists($pixelsR, $this->indentR))
				$indentR = $this->indentR[40];
			else
				$indentR = $this->indentR[$pixelsR];
		}
		if($indentL || $indentR)
			$text = "\{$indentL$indentR\n$text}\n\n\\li0\\ri0\n";
			$text = str_replace("WIKINDX_SPACE", " ", $text);
		return $text;
	}
/*
* deal with <P> parameters
* @author Mark Grimshaw
* @version 1
* @param string
* @param string
* @return string
*/
	function paraStyle($param, $text)
	{
		$text = str_replace(" ", "&nbsp;", $text);
		$justify = $pixelsR = $pixelsL = $indentL = $indentR = FALSE;
		$justify = trim($param);
// Although it works in OO.org, for Word, justification needs to be outside {...} and therefore not mixed up with bold, italics etc.  so deal with it last.
		if($justify)
		{
			if(!array_key_exists($justify, $this->justify))
				$justify = $this->justify['justify'];
			else
				$justify = $this->justify[$justify];
			$text = "$justify\n$text\n";
		}
		if($pixelsL)
		{
			if(!array_key_exists($pixelsL, $this->indentL))
				$indentL = $this->indentL[40];
			else
				$indentL = $this->indentL[$pixelsL];
		}
		if($pixelsR)
		{
			if(!array_key_exists($pixelsR, $this->indentR))
				$indentR = $this->indentR[40];
			else
				$indentR = $this->indentR[$pixelsR];
		}
		if($indentL || $indentR)
			$text = "$indentL$indentR\n$text\n\\par\n\\li0\\ri0\n";
		return $text;
	}
/*
* indent long quotations
* @author Mark Grimshaw
* @version 1
* @param string
* @param integer
* @param integer
* @param BOOLEAN
* @return string
*/
	function indentQuotations($text, $numWords, $fontSize, $keepQuoteMarks)
	{
		$this->quoteNumWords = $numWords;
		$this->keepQuoteMarks = $keepQuoteMarks;
		if(array_key_exists($fontSize, $this->fontSizes))
			$this->quoteFontSize = $this->fontSizes[++$fontSize];
		else
			$this->quoteFontSize = $this->fontSizes[1]; // font size 11/22
// Get quotation marks for this localization
		if(!$languageDir = $this->session->getVar("setup_language"))
			$languageDir = "en";
		include_once("languages/$languageDir/CONSTANTS.php");
		$class = "CONSTANTS_" . $languageDir;
		$this->wikindxLanguageClass = new $class();
		if(isset($this->wikindxLanguageClass->startQuotation) && $this->wikindxLanguageClass->startQuotation)
			$qms = preg_quote($this->wikindxLanguageClass->startQuotation);
		else
			$qms = "\"";
		$this->qms = $qms;
		if(isset($this->wikindxLanguageClass->endQuotation) && $this->wikindxLanguageClass->endQuotation)
			$qme = preg_quote($this->wikindxLanguageClass->endQuotation);
		else
			$qme = "\"";
		$this->qme = $qme;
		return preg_replace_callback("/($qms)(.*)($qme)(.*)(\[cite\].*\[\/cite\])(.*)(\[footnote\].*\[\/footnote\]){0,}([.,:;?!])/Usi", array($this, "indentQ"), $text);
	}
/*
* callback for indent long quotations
* @author Mark Grimshaw
* @version 1
* @param array
* @return string
*/
	function indentQ($matchArray)
	{
// [1] => initial quote marker
// [2] => quoted text
// [3] => final quote marker
// [4] => intervening text (should be a single space)
// [5] => citation
// [6] => intervening text
// [7] => tags such as footnotes etc.
// [8] => punctuation immediately following citation

//print "HERE: "; print_r($matchArray); print "<P>";
//print $matchArray[4]; print "<P>";
// No indentation if this is part of a footnote...
		if(strpos($matchArray[4], '[/footnote]'))
			return $matchArray[0];
// are there quotes within the quote?
		if(strpos($matchArray[4], $this->qme))
		{
			$split = explode($this->qme, $matchArray[4]);
//print_r($split); print "<P>";
			$lastElement = array_pop($split);
			$matchArray[4] = $lastElement;
			$matchArray[2] .= $matchArray[3] . join($this->qme, $split);
//print $matchArray[2]; print "<P>";
//print $matchArray[4]; die;
		}
		if(str_word_count($matchArray[2]) >= $this->quoteNumWords)
		{
			if(trim($matchArray[6])) // intervening text
			{
				$trail = trim($matchArray[6]) . $matchArray[7] . $matchArray[8];
				$removeSpace = '';
			}
			else
			{
				$trail = $matchArray[6] . $matchArray[7] . trim($matchArray[8]);
				$removeSpace = '__WIKINDX__QUOTEINDENTREMOVESPACE__';
			}
			$fs = $this->quoteFontSize . $this->lineSpacingIndentQ;
			$start = "\n__WIKINDX__NEWLINEPAR__{\\fs$fs __WIKINDX__NEWLINEPAR__\\li720\\ri720";
			$codaA = $matchArray[4] . $matchArray[5] . $trail . 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__$removeSpace";
			$codaB = 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__" . $matchArray[4] . $matchArray[5] . 
				$trail . $removeSpace;
			$codaC = $matchArray[4] . $matchArray[5] . 
				"__WIKINDX__NEWLINEPAR__}__WIKINDX__NEWLINEPAR__" . $trail . $removeSpace;
			if($this->keepQuoteMarks)
			{
				if($matchArray[4] == ' ')
				{
					if(trim($matchArray[6])) // intervening text
						return $start . $this->qms . $matchArray[2] . $this->qme . $codaC;
					else
						return $start . $this->qms . $matchArray[2] . $this->qme . $codaA;
				}
				else // intervening text
					return $start . $this->qms . $matchArray[2] . $this->qme . $codaB;

			}
			else
			{
				if($matchArray[4] == ' ')
				{
					if(trim($matchArray[6])) // intervening text
						return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
						'__WIKINDX__QUOTEINDENTDONE__' . $codaC;
					else
						return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
						'__WIKINDX__QUOTEINDENTDONE__' . $codaA;
				}
				else // intervening text
					return $start . '__WIKINDX__QUOTEINDENTDONE__' . $matchArray[2] . 
					'__WIKINDX__QUOTEINDENTDONE__' . $codaB;

			}
		}
		return $matchArray[0];
//		unset($matchArray[0]);
//		return join('', $matchArray);
	}
/*
* parse [footnote]...[/footnote]
* @author Mark Grimshaw
* @version 1
* @param string
* @param array
* @param array
* @return string
*/
	function parseFootnotes($text, $styleArray, $footnoteOffsetIds, $fontSize)
	{
//print_r($styleArray);
		if(array_key_exists($fontSize, $this->fontSizes))
			$this->fontSizeFt = $this->fontSizes[$fontSize];
		else
			$this->fontSizeFt = $this->fontSizes[0]; // font size 9/18
		$this->styleArray = $styleArray;
		if(!empty($footnoteOffsetIds)) // endnotes same IDs and there are existing cite tags being used
			$this->footnoteOffsetIds = $footnoteOffsetIds;
		return preg_replace_callback("/\[footnote\](.*)\[\/footnote\]([.,:;?!]*\s*)/Usi", 
			array($this, "footnotes"), $text);
	}
/*
* callback for parsing [footnote]...[/footnote]
* @author Mark Grimshaw
* @version 1
* @param array
* @return string
*/
	function footnotes($matchArray)
	{
		$text = $matchArray[1];
		$id = '\\chftn';
		if(!$this->styleArray['citationStyle']) // in-text citations so footnotes here really are footnotes
			$ft = "\\footnote";
		else // endnotes and footnotes
		{
			if(!$this->styleArray['endnoteStyle']) // endnotes incrementing
				$ft = "\\footnote\\ftnalt";
			else if($this->styleArray['endnoteStyle'] == 1) // endnotes same IDs
			{
				$ft = "\\footnote\\ftnalt";
				$id = array_shift($this->footnoteOffsetIds);
			}
			else	// '2' == footnotes incrementing
				$ft = "\\footnote";
		}
		$preInText = $this->styleArray['firstCharsEndnoteInText'];
		$postInText = $this->styleArray['lastCharsEndnoteInText'];
		$preId = $this->styleArray['firstCharsEndnoteID'];
		if($postInText && preg_match("/[.,:;?!]\s*/", $matchArray[2], $null))
			$matchArray[2] = '';
		$postId = $this->styleArray['lastCharsEndnoteID'];
		if($this->styleArray['formatEndnoteInText'] == 1) // superscript
		{
			$preInTextFormat = "{\\super";
			$postInTextFormat = '}';
		}
		else if($this->styleArray['formatEndnoteInText'] == 2) // subscript
		{
			$preInTextFormat = "{\\sub";
			$postInTextFormat = '}';
		}
		else
		{
			$preInTextFormat = '';
			$postInTextFormat = '';
		}
		if($this->styleArray['formatEndnoteID'] == 1) // superscript
		{
			$preIDFormat = "{\\super";
			$postIDFormat = '}';
		}
		else if($this->styleArray['formatEndnoteID'] == 2) // subscript
		{
			$preIDFormat = "{\\sub";
			$postIDFormat = '}';
		}
		else
		{
			$preIDFormat = '';
			$postIDFormat = '';
		}
		if($this->session->getVar('exportPaper_indentFt') == 'indentAll')
			$ftf = "\\li720 ";
		else if($this->session->getVar('exportPaper_indentFt') == 'indentFL')
			$ftf = "\\fi720 ";
		else if($this->session->getVar('exportPaper_indentFt') == 'indentNotFL')
			$ftf = "\\li720\\fi-720 ";
		else
			$ftf = "\\li1\\fi1 ";
		$ftf .= "\\fs" . $this->fontSizeFt;
		if($this->session->getVar('exportPaper_spaceFt') == 'oneHalfSpace')
			$ftf = "\\pard\\plain $ftf\\sl360\\slmult1 ";
		else if($this->session->getVar('exportPaper_spaceFt') == 'doubleSpace')
			$ftf = "\\pard\\plain $ftf\\sl480\\slmult1 ";
		else
			$ftf = "\\pard\\plain$ftf";
		if(($this->styleArray['endnoteStyle'] == 1)
			 && array_key_exists('sameIdOrderBib', $this->styleArray)) // endnotes same IDs, bibliography order
		{
			$this->footnoteText .= "$ftf\\qj$preIDFormat$preId{\\cs2 $id}" . $postId . $postIDFormat . 
				$matchArray[1] . "\\par\n" . $postInTextFormat . $matchArray[2];
			return "\n$preInTextFormat{" . "$preInText$id$postInText\n}}";
		}
		return "\n$preInTextFormat$preInText{\\cs2 $id\n{" . 
			"$ft$ftf\\qj" . "$preIDFormat$preId{\\cs2 $id}" . $postId . "$postIDFormat{" . 
			$text . "}}}$postInText\n" . $postInTextFormat . $matchArray[2];
	}
}
?>
